@@ -9,24 +9,39 @@
 import collections
 import datetime
 import enum
-import math
+import json
 import warnings
-from typing import List, Tuple
+from typing import Dict, Generator, List, Optional, TextIO, Tuple, TypeVar, Union
 
+from .tz import Mountain
 from . import iersdata
 
-
-from .tzinfo_us import Mountain, HOUR
+HOUR = datetime.timedelta(seconds=3600)
+DateOrDatetime = TypeVar("DateOrDatetime", datetime.date, datetime.datetime)
+T = TypeVar("T")  # pylint: disable=invalid-name
 
 
-def _date(dt):
+def require(x: Optional[T]) -> T:
+    """Assert that an Optional value is not None, and then return the value,
+    giving a hint to the type system."""
+    assert x is not None
+    return x
+
+
+def _removeprefix(s: str, p: str) -> str:
+    if s.startswith(p):
+        return s[len(p) :]
+    return s
+
+
+def _date(dt: DateOrDatetime) -> datetime.date:
     """Return the date object itself, or the date property of a datetime"""
     if isinstance(dt, datetime.datetime):
         return dt.date()
     return dt
 
 
-def maybe_warn_update(dt):  # pragma no cover
+def _maybe_warn_update(dt: datetime.date) -> None:
     """Maybe print a notice to run updateiers, if it seems useful to do so."""
     # We already know this date is not covered.
     # If the date is less than 330 days after today, there should be (possibly)
@@ -38,21 +53,29 @@ def maybe_warn_update(dt):  # pragma no cover
         )
 
 
-def get_dut1(dt):
+def get_dut1(dt: DateOrDatetime, *, warn_outdated: bool = True) -> float:
     """Return the DUT1 number for the given timestamp"""
-    dt = _date(dt)
-    i = (dt - iersdata.DUT1_DATA_START).days
+    date = _date(dt)
+    i = (date - iersdata.DUT1_DATA_START).days
     if i < 0:
         v = iersdata.DUT1_OFFSETS[0]
     elif i >= len(iersdata.DUT1_OFFSETS):
-        maybe_warn_update(dt)
+        if warn_outdated:
+            _maybe_warn_update(dt)
         v = iersdata.DUT1_OFFSETS[-1]
     else:
         v = iersdata.DUT1_OFFSETS[i]
     return (ord(v) - ord("k")) / 10.0
 
 
-def isls(t):
+def isly(year: int) -> bool:
+    """Return True if the year is a leap year"""
+    d1 = datetime.date(year, 1, 1)
+    d2 = d1 + datetime.timedelta(days=365)
+    return d1.year == d2.year
+
+
+def isls(t: DateOrDatetime) -> bool:
     """Return True if a leap second occurs at the end of this month"""
     dut1_today = get_dut1(t)
     month_today = t.month
@@ -62,27 +85,33 @@ def isls(t):
     return dut1_today * dut1_next_month < 0
 
 
-def isdst(t, tz=Mountain):
-    """Return true if daylight saving time is active at the given moment"""
-    t = datetime.datetime(t.year, t.month, t.day)
+def isdst(t: datetime.date, tz: datetime.tzinfo = Mountain) -> bool:
+    """Return true if daylight saving time is active at the start of the given UTC day"""
+    t = datetime.datetime(t.year, t.month, t.day, tzinfo=datetime.timezone.utc)
     return bool(t.astimezone(tz).dst())
 
 
-def first_sunday_in_month(y, m):
+def first_sunday_on_or_after(dt: DateOrDatetime) -> DateOrDatetime:
+    """Return the first sunday on or after the reference time"""
+    days_to_go = 6 - dt.weekday()
+    if days_to_go:
+        return dt + datetime.timedelta(days_to_go)
+    return dt
+
+
+def first_sunday_in_month(y: int, m: int) -> datetime.date:
     """Find the first sunday in a given month"""
-    for md in range(1, 8):
-        d = datetime.date(y, m, md)
-        if d.weekday() == 6:
-            return d
-    raise RuntimeError("Impossible week without Sunday")  # pragma no cover
+    return first_sunday_on_or_after(datetime.datetime(y, m, 1))
 
 
-def is_dst_change_day(t):
+def is_dst_change_day(t: datetime.date) -> bool:
     """Return True if the day is a DST change day"""
     return isdst(t) != isdst(t + datetime.timedelta(1))
 
 
-def get_dst_change_hour(t, tz=Mountain):
+def get_dst_change_hour(
+    t: DateOrDatetime, tz: datetime.tzinfo = Mountain
+) -> Optional[int]:
     """Return the hour when DST changes"""
     lt0 = datetime.datetime(t.year, t.month, t.day, hour=0, tzinfo=tz)
     dst0 = lt0.dst()
@@ -91,25 +120,27 @@ def get_dst_change_hour(t, tz=Mountain):
         dst1 = lt1.dst()
         if dst0 != dst1:
             return i - 1
-    return None  # pragma no cover
+    return None
 
 
-def get_dst_change_date_and_row(d):
+def get_dst_change_date_and_row(
+    d: DateOrDatetime,
+) -> Tuple[Optional[datetime.date], Optional[int]]:
     """Classify DST information for the WWVB phase modulation signal"""
     if isdst(d):
         n = first_sunday_in_month(d.year, 11)
-        for offset in (-28, -21, -14, -7, 0, 7, 14, 21):
+        for offset in range(-28, 28, 7):  # pragma no branch
             d1 = n + datetime.timedelta(days=offset)
             if is_dst_change_day(d1):
                 return d1, (offset + 28) // 7
     else:
         m = first_sunday_in_month(d.year + (d.month > 3), 3)
-        for offset in (0, 7, 14, 21, 28, 35, 42, 49):
+        for offset in range(0, 52, 7):
             d1 = m + datetime.timedelta(days=offset)
             if is_dst_change_day(d1):
                 return d1, offset // 7
 
-    return None, None  # pragma no coverage
+    return None, None
 
 
 # "Table 8", likely with transcrption errors
@@ -181,7 +212,7 @@ dsttable = [
 ]
 
 
-def lfsr_gen(x):
+def lfsr_gen(x: List[int]) -> None:
     """Generate the 127-bit sequence used in the extended 6-minute codes
     except generate 255 bits so that we can simply use any range of [x:x+127]
     bits"""
@@ -209,7 +240,7 @@ ftw = [
 ]
 
 
-def get_dst_next(d, tz=Mountain):
+def get_dst_next(d: DateOrDatetime, tz: datetime.tzinfo = Mountain) -> int:
     """Find the "dst next" value for the phase modulation signal"""
     dst_now = isdst(d)  # dst_on[1]
     dst_midwinter = isdst(datetime.datetime(d.year, 1, 1))
@@ -217,7 +248,7 @@ def get_dst_next(d, tz=Mountain):
 
     if dst_midwinter and dst_midsummer:  # pragma no coverage
         return 0b101111
-    if not (dst_midwinter or dst_midsummer):  # pragma no coverage
+    if not (dst_midwinter or dst_midsummer):
         return 0b000111
 
     # Are we in NZ or something?
@@ -250,12 +281,12 @@ SYNC_T = 0x768
 SYNC_M = 0x1A3A
 
 
-def extract_bit(v, p):
+def extract_bit(v: int, p: int) -> bool:
     """Extract bit 'p' from integer 'v' as a bool"""
     return bool((v >> p) & 1)
 
 
-def hamming_parity(value: int):
+def hamming_parity(value: int) -> int:
     """Compute the "hamming parity" of a 26-bit number, such as the minute-of-century [See Enhanced WWVB Broadcast Format 4.3]"""
     parity = 0
     for i in range(4, -1, -1):
@@ -285,32 +316,68 @@ dst_ls_lut = [
     0b11111,
 ]
 
-_WWVBMinute = collections.namedtuple("_WWVBMinute", "year days hour min dst ut1 ls")
+_WWVBMinute = collections.namedtuple("_WWVBMinute", "year days hour min dst ut1 ls ly")
 
 
 class WWVBMinute(_WWVBMinute):
     """Uniquely identifies a minute of time in the WWVB system. To use ut1 and ls information from IERS, create a WWVBMinuteIERS value instead."""
 
-    def __new__(
-        cls, year, days, hour, minute, dst=None, ut1=None, ls=None
-    ):  # pylint: disable=too-many-arguments
+    year: int
+    hour: int
+    minute: int
+    dst: int
+    ut1: int
+    ly: bool
+    ls: bool
+
+    epoch: int = 1970
+
+    def __new__(  # pylint: disable=too-many-arguments
+        cls,
+        year: int,
+        days: int,
+        hour: int,
+        minute: int,
+        dst: Optional[int] = None,
+        ut1: Optional[int] = None,
+        ls: Optional[bool] = None,
+        ly: Optional[bool] = None,
+    ) -> "WWVBMinute":
         """Construct a WWVBMinute"""
         if dst is None:
             dst = cls.get_dst(year, days)
-        if dst not in (0, 1, 2, 3):  # pragma no coverage
+        if dst not in (0, 1, 2, 3):
             raise ValueError("dst value should be 0..3")
         if ut1 is None and ls is None:
-            ut1, ls = cls.get_dut1_info(year, days)
-        elif ut1 is None or ls is None:  # pragma no coverage
+            ut1, ls = cls._get_dut1_info(year, days)
+        elif ut1 is None or ls is None:
             raise ValueError("sepecify both ut1 and ls or neither one")
-        if year < 70:
-            year = year + 2000
-        elif year < 100:
-            year = year + 1900
-        return _WWVBMinute.__new__(cls, year, days, hour, minute, dst, ut1, ls)
+        year = cls.full_year(year)
+        if ly is None:
+            ly = isly(year)
+        return _WWVBMinute.__new__(cls, year, days, hour, minute, dst, ut1, ls, ly)
+
+    @classmethod
+    def full_year(cls, year: int) -> int:
+        """Convert a (possibly two-digit) year to a full year.
+
+        If the argument is above 100, it is assumed to be a full year.
+        Otherwise, the intuitive method is followed: Say the epoch is 1970,
+        then 70..99 means 1970..99 and 00..69 means 2000..2069.
+
+        To actually use a different epoch, derive a class from WWVBMinute (or
+        WWVBMinuteIERS) and give it a different epoch property.  Then, create
+        instances of that class instead of WWVBMinute.
+        """
+        century = cls.epoch // 100 * 100
+        if year < (cls.epoch % 100):
+            return year + century + 100
+        if year < 100:
+            return year + century
+        return year
 
     @staticmethod
-    def get_dst(year, days):
+    def get_dst(year: int, days: int) -> int:
         """Get the 2-bit WWVB DST value for the given day"""
         d0 = datetime.datetime(year, 1, 1) + datetime.timedelta(days - 1)
         d1 = d0 + datetime.timedelta(1)
@@ -318,20 +385,11 @@ class WWVBMinute(_WWVBMinute):
         dst1 = isdst(d1)
         return dst1 * 2 + dst0
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Implement str()"""
-        return "year=%4d days=%.3d hour=%.2d min=%.2d dst=%d ut1=%d ly=%d ls=%d" % (
-            self.year,
-            self.days,
-            self.hour,
-            self.min,
-            self.dst,
-            self.ut1,
-            self.is_ly(),
-            self.ls,
-        )
+        return f"year={self.year:4d} days={self.days:03d} hour={self.hour:02d} min={self.min:02d} dst={self.dst} ut1={self.ut1} ly={int(self.ly)} ls={int(self.ls)}"
 
-    def as_datetime_utc(self):
+    def as_datetime_utc(self) -> datetime.datetime:
         """Convert to a UTC datetime"""
         d = datetime.datetime(self.year, 1, 1, tzinfo=datetime.timezone.utc)
         d += datetime.timedelta(self.days - 1, self.hour * 3600 + self.min * 60)
@@ -339,9 +397,12 @@ class WWVBMinute(_WWVBMinute):
 
     as_datetime = as_datetime_utc
 
-    def as_datetime_local(self, standard_time_offset=7 * 3600, dst_observed=True):
+    def as_datetime_local(
+        self, standard_time_offset: int = 7 * 3600, dst_observed: bool = True
+    ) -> datetime.datetime:
         """Convert to a local datetime according to the DST bits"""
         u = self.as_datetime_utc()
+        offset = datetime.timedelta(seconds=-standard_time_offset)
         d = u - datetime.timedelta(seconds=standard_time_offset)
         if not dst_observed:
             dst = False
@@ -356,25 +417,25 @@ class WWVBMinute(_WWVBMinute):
         else:  # self.dst == 0b00
             dst = False
         if dst:
-            d += datetime.timedelta(seconds=3600)
-        return d
+            offset += datetime.timedelta(seconds=3600)
+        return u.astimezone(datetime.timezone(offset))
 
-    def is_ly(self):
+    def is_ly(self) -> bool:
         """Return True if minute is during a leap year"""
-        d = datetime.datetime(self.year, 1, 1) + datetime.timedelta(365)
-        return d.year == self.year
+        warnings.warn("Deprecated, use ly property instead", DeprecationWarning)
+        return self.ly
 
-    def is_end_of_month(self):
+    def _is_end_of_month(self) -> bool:
         """Return True if minute is the last minute in a month"""
         d = self.as_datetime()
         e = d + datetime.timedelta(1)
         return d.month != e.month
 
-    def minute_length(self):
+    def minute_length(self) -> int:
         """Return the length of the minute, 60, 61, or (theoretically) 59 seconds"""
         if not self.ls:
             return 60
-        if not self.is_end_of_month():
+        if not self._is_end_of_month():
             return 60
         if self.hour != 23 or self.min != 59:
             return 60
@@ -382,7 +443,7 @@ class WWVBMinute(_WWVBMinute):
             return 59
         return 61
 
-    def as_timecode(self):
+    def as_timecode(self) -> "WWVBTimecode":
         """Fill a WWVBTimecode structure representing this minute.  Fills both the amplitude and phase codes."""
         t = WWVBTimecode(self.minute_length())
 
@@ -392,7 +453,7 @@ class WWVBMinute(_WWVBMinute):
         return t
 
     @property
-    def leap_sec(self):
+    def leap_sec(self) -> int:
         """Return the 2-bit leap_sec value used by the PM code"""
         if not self.ls:
             return 0
@@ -401,7 +462,7 @@ class WWVBMinute(_WWVBMinute):
         return 2
 
     @property
-    def minute_of_century(self):
+    def minute_of_century(self) -> int:
         """Return the minute of the century"""
         century = (self.year // 100) * 100
         # note: This relies on timedelta seconds never including leapseconds!
@@ -415,7 +476,7 @@ class WWVBMinute(_WWVBMinute):
             // 60
         )
 
-    def fill_am_timecode(self, t):
+    def fill_am_timecode(self, t: "WWVBTimecode") -> None:
         """Fill the amplitude (AM) portion of a timecode object"""
         for i in [0, 9, 19, 29, 39, 49]:
             t.am[i] = AmplitudeModulation.MARK
@@ -425,19 +486,21 @@ class WWVBMinute(_WWVBMinute):
             t.am[60] = AmplitudeModulation.MARK
         for i in [4, 10, 11, 14, 20, 21, 24, 34, 35, 44, 54]:
             t.am[i] = AmplitudeModulation.ZERO
-        t.put_am_bcd(self.min, 1, 2, 3, 5, 6, 7, 8)
-        t.put_am_bcd(self.hour, 12, 13, 15, 16, 17, 18)
-        t.put_am_bcd(self.days, 22, 23, 25, 26, 27, 28, 30, 31, 32, 33)
+        t._put_am_bcd(self.min, 1, 2, 3, 5, 6, 7, 8)
+        t._put_am_bcd(self.hour, 12, 13, 15, 16, 17, 18)
+        t._put_am_bcd(self.days, 22, 23, 25, 26, 27, 28, 30, 31, 32, 33)
         ut1_sign = self.ut1 >= 0
         t.am[36] = t.am[38] = AmplitudeModulation(ut1_sign)
         t.am[37] = AmplitudeModulation(not ut1_sign)
-        t.put_am_bcd(abs(self.ut1) // 100, 40, 41, 42, 43)
-        t.put_am_bcd(self.year, 45, 46, 47, 48, 50, 51, 52, 53)
-        t.am[55] = AmplitudeModulation(self.is_ly())
+        t._put_am_bcd(abs(self.ut1) // 100, 40, 41, 42, 43)
+        t._put_am_bcd(
+            self.year, 45, 46, 47, 48, 50, 51, 52, 53
+        )  # Implicitly discards all but lowest 2 digits of year
+        t.am[55] = AmplitudeModulation(self.ly)
         t.am[56] = AmplitudeModulation(self.ls)
-        t.put_am_bcd(self.dst, 57, 58)
+        t._put_am_bcd(self.dst, 57, 58)
 
-    def fill_pm_timecode_extended(self, t):
+    def fill_pm_timecode_extended(self, t: "WWVBTimecode") -> None:
         """During minutes 10..15 and 40..45, the amplitude signal holds 'extended information'"""
         assert 10 <= self.min < 16 or 40 <= self.min < 46
         minno = self.min % 10
@@ -470,89 +533,95 @@ class WWVBMinute(_WWVBMinute):
 
         offset = minno * 60
         for i in range(60):
-            t.put_pm_bit(i, full_seq[i + offset])
+            t._put_pm_bit(i, full_seq[i + offset])
 
-    def fill_pm_timecode_regular(self, t):  # pylint: disable=too-many-statements
-        """Not during minutes 10..15 and 40..45, the amplitude signal holds 'regular information'"""
-        t.put_pm_bin(0, 13, SYNC_T)
+    def fill_pm_timecode_regular(  # pylint: disable=too-many-statements
+        self, t: "WWVBTimecode"
+    ) -> None:
+        """Except during minutes 10..15 and 40..45, the amplitude signal holds 'regular information'"""
+        t._put_pm_bin(0, 13, SYNC_T)
 
         moc = self.minute_of_century
         leap_sec = self.leap_sec
         dst_on = self.dst
         dst_ls = dst_ls_lut[dst_on | (leap_sec << 2)]
         dst_next = get_dst_next(self.as_datetime())
-        t.put_pm_bin(13, 5, hamming_parity(moc))
-        t.put_pm_bit(18, extract_bit(moc, 25))
-        t.put_pm_bit(19, extract_bit(moc, 0))
-        t.put_pm_bit(20, extract_bit(moc, 24))
-        t.put_pm_bit(21, extract_bit(moc, 23))
-        t.put_pm_bit(22, extract_bit(moc, 22))
-        t.put_pm_bit(23, extract_bit(moc, 21))
-        t.put_pm_bit(24, extract_bit(moc, 20))
-        t.put_pm_bit(25, extract_bit(moc, 19))
-        t.put_pm_bit(26, extract_bit(moc, 18))
-        t.put_pm_bit(27, extract_bit(moc, 17))
-        t.put_pm_bit(28, extract_bit(moc, 16))
-        t.put_pm_bit(29, False)  # Reserved
-        t.put_pm_bit(30, extract_bit(moc, 15))
-        t.put_pm_bit(31, extract_bit(moc, 14))
-        t.put_pm_bit(32, extract_bit(moc, 13))
-        t.put_pm_bit(33, extract_bit(moc, 12))
-        t.put_pm_bit(34, extract_bit(moc, 11))
-        t.put_pm_bit(35, extract_bit(moc, 10))
-        t.put_pm_bit(36, extract_bit(moc, 9))
-        t.put_pm_bit(37, extract_bit(moc, 8))
-        t.put_pm_bit(38, extract_bit(moc, 7))
-        t.put_pm_bit(39, True)  # Reserved
-        t.put_pm_bit(40, extract_bit(moc, 6))
-        t.put_pm_bit(41, extract_bit(moc, 5))
-        t.put_pm_bit(42, extract_bit(moc, 4))
-        t.put_pm_bit(43, extract_bit(moc, 3))
-        t.put_pm_bit(44, extract_bit(moc, 2))
-        t.put_pm_bit(45, extract_bit(moc, 1))
-        t.put_pm_bit(46, extract_bit(moc, 0))
-        t.put_pm_bit(47, extract_bit(dst_ls, 4))
-        t.put_pm_bit(48, extract_bit(dst_ls, 3))
-        t.put_pm_bit(49, True)  # Notice
-        t.put_pm_bit(50, extract_bit(dst_ls, 2))
-        t.put_pm_bit(51, extract_bit(dst_ls, 1))
-        t.put_pm_bit(52, extract_bit(dst_ls, 0))
-        t.put_pm_bit(53, extract_bit(dst_next, 5))
-        t.put_pm_bit(54, extract_bit(dst_next, 4))
-        t.put_pm_bit(55, extract_bit(dst_next, 3))
-        t.put_pm_bit(56, extract_bit(dst_next, 2))
-        t.put_pm_bit(57, extract_bit(dst_next, 1))
-        t.put_pm_bit(58, extract_bit(dst_next, 0))
+        t._put_pm_bin(13, 5, hamming_parity(moc))
+        t._put_pm_bit(18, extract_bit(moc, 25))
+        t._put_pm_bit(19, extract_bit(moc, 0))
+        t._put_pm_bit(20, extract_bit(moc, 24))
+        t._put_pm_bit(21, extract_bit(moc, 23))
+        t._put_pm_bit(22, extract_bit(moc, 22))
+        t._put_pm_bit(23, extract_bit(moc, 21))
+        t._put_pm_bit(24, extract_bit(moc, 20))
+        t._put_pm_bit(25, extract_bit(moc, 19))
+        t._put_pm_bit(26, extract_bit(moc, 18))
+        t._put_pm_bit(27, extract_bit(moc, 17))
+        t._put_pm_bit(28, extract_bit(moc, 16))
+        t._put_pm_bit(29, False)  # Reserved
+        t._put_pm_bit(30, extract_bit(moc, 15))
+        t._put_pm_bit(31, extract_bit(moc, 14))
+        t._put_pm_bit(32, extract_bit(moc, 13))
+        t._put_pm_bit(33, extract_bit(moc, 12))
+        t._put_pm_bit(34, extract_bit(moc, 11))
+        t._put_pm_bit(35, extract_bit(moc, 10))
+        t._put_pm_bit(36, extract_bit(moc, 9))
+        t._put_pm_bit(37, extract_bit(moc, 8))
+        t._put_pm_bit(38, extract_bit(moc, 7))
+        t._put_pm_bit(39, True)  # Reserved
+        t._put_pm_bit(40, extract_bit(moc, 6))
+        t._put_pm_bit(41, extract_bit(moc, 5))
+        t._put_pm_bit(42, extract_bit(moc, 4))
+        t._put_pm_bit(43, extract_bit(moc, 3))
+        t._put_pm_bit(44, extract_bit(moc, 2))
+        t._put_pm_bit(45, extract_bit(moc, 1))
+        t._put_pm_bit(46, extract_bit(moc, 0))
+        t._put_pm_bit(47, extract_bit(dst_ls, 4))
+        t._put_pm_bit(48, extract_bit(dst_ls, 3))
+        t._put_pm_bit(49, True)  # Notice
+        t._put_pm_bit(50, extract_bit(dst_ls, 2))
+        t._put_pm_bit(51, extract_bit(dst_ls, 1))
+        t._put_pm_bit(52, extract_bit(dst_ls, 0))
+        t._put_pm_bit(53, extract_bit(dst_next, 5))
+        t._put_pm_bit(54, extract_bit(dst_next, 4))
+        t._put_pm_bit(55, extract_bit(dst_next, 3))
+        t._put_pm_bit(56, extract_bit(dst_next, 2))
+        t._put_pm_bit(57, extract_bit(dst_next, 1))
+        t._put_pm_bit(58, extract_bit(dst_next, 0))
         if len(t.phase) > 59:
-            t.put_pm_bit(59, PhaseModulation.ZERO)
+            t._put_pm_bit(59, PhaseModulation.ZERO)
         if len(t.phase) > 60:
-            t.put_pm_bit(60, PhaseModulation.ZERO)
+            t._put_pm_bit(60, PhaseModulation.ZERO)
 
-    def fill_pm_timecode(self, t):
+    def fill_pm_timecode(self, t: "WWVBTimecode") -> None:
         """Fill the phase portion of a timecode object"""
         if 10 <= self.min < 16 or 40 <= self.min < 46:
             self.fill_pm_timecode_extended(t)
         else:
             self.fill_pm_timecode_regular(t)
 
-    def next_minute(self, newut1=None, newls=None):
+    def next_minute(
+        self, newut1: Optional[int] = None, newls: Optional[bool] = None
+    ) -> "WWVBMinute":
         """Return an object representing the next minute"""
         d = self.as_datetime() + datetime.timedelta(minutes=1)
         return self.from_datetime(d, newut1, newls, self)
 
-    def previous_minute(self, newut1=None, newls=None):
+    def previous_minute(
+        self, newut1: Optional[int] = None, newls: Optional[bool] = None
+    ) -> "WWVBMinute":
         """Return an object representing the previous minute"""
         d = self.as_datetime() - datetime.timedelta(minutes=1)
         return self.from_datetime(d, newut1, newls, self)
 
     @classmethod
-    def get_dut1_info(
-        cls, year, days, old_time=None
-    ):  # pylint: disable=unused-argument
+    def _get_dut1_info(  # pylint: disable=unused-argument
+        cls: type, year: int, days: int, old_time: "Optional[WWVBMinute]" = None
+    ) -> Tuple[int, bool]:
         """Return the DUT1 information for a given day, possibly propagating information from a previous timestamp"""
         if old_time is not None:
             if old_time.minute_length() != 60:
-                newls = 0
+                newls = False
                 if old_time.ut1 < 0:
                     newut1 = old_time.ut1 + 1000
                 else:
@@ -561,33 +630,48 @@ class WWVBMinute(_WWVBMinute):
                 newls = old_time.ls
                 newut1 = old_time.ut1
             return newut1, newls
-        return 0, 0
+        return 0, False
 
     @classmethod
-    def fromstring(cls, s):
+    def fromstring(cls, s: str) -> "WWVBMinute":
         """Construct a WWVBMinute from a string representation created by print_timecodes"""
-        if s.startswith("WWVB timecode: "):
-            s = s[len("WWVB timecode: ") :]
-        d = {}
+        s = _removeprefix(s, "WWVB timecode: ")
+        d: Dict[str, int] = {}
         for part in s.split():
             k, v = part.split("=")
             if k == "min":
                 k = "minute"
             d[k] = int(v)
-        if "ly" in d:
-            d.pop("ly")
-        return cls(**d)
+        year = d.pop("year")
+        days = d.pop("days")
+        hour = d.pop("hour")
+        minute = d.pop("minute")
+        dst: Optional[int] = d.pop("dst", None)
+        ut1: Optional[int] = d.pop("ut1", None)
+        ls = d.pop("ls", None)
+        d.pop("ly", None)
+        if d:
+            raise ValueError(f"Invalid options: {d}")
+        return cls(year, days, hour, minute, dst, ut1, None if ls is None else bool(ls))
 
     @classmethod
-    def from_datetime(cls, d, newut1=None, newls=None, old_time=None):
+    def from_datetime(
+        cls,
+        d: datetime.datetime,
+        newut1: Optional[int] = None,
+        newls: Optional[bool] = None,
+        old_time: Optional["WWVBMinute"] = None,
+    ) -> "WWVBMinute":
         """Construct a WWVBMinute from a datetime, possibly specifying ut1/ls data or propagating it from an old time"""
         u = d.utctimetuple()
         if newls is None and newut1 is None:
-            newut1, newls = cls.get_dut1_info(u.tm_year, u.tm_yday, old_time)
+            newut1, newls = cls._get_dut1_info(u.tm_year, u.tm_yday, old_time)
         return cls(u.tm_year, u.tm_yday, u.tm_hour, u.tm_min, ut1=newut1, ls=newls)
 
     @classmethod
-    def from_timecode_am(cls, t):  # pylint: disable=too-many-return-statements
+    def from_timecode_am(  # pylint: disable=too-many-return-statements
+        cls, t: "WWVBTimecode"
+    ) -> Optional["WWVBMinute"]:
         """Construct a WWVBMinute from a WWVBTimecode"""
         for i in (0, 9, 19, 29, 39, 49, 59):
             if t.am[i] != AmplitudeModulation.MARK:
@@ -599,59 +683,50 @@ class WWVBMinute(_WWVBMinute):
             return None
         if t.am[36] != t.am[38]:
             return None
-        ok, minute = t.get_am_bcd(1, 2, 3, 5, 6, 7, 8)
-        if not ok:
+        minute = t._get_am_bcd(1, 2, 3, 5, 6, 7, 8)
+        if minute is None:
             return None
-        ok, hour = t.get_am_bcd(12, 13, 15, 16, 17, 18)
-        if not ok:
+        hour = t._get_am_bcd(12, 13, 15, 16, 17, 18)
+        if hour is None:
             return None
-        ok, days = t.get_am_bcd(22, 23, 25, 26, 27, 28, 30, 31, 32, 33)
-        if not ok:
+        days = t._get_am_bcd(22, 23, 25, 26, 27, 28, 30, 31, 32, 33)
+        if days is None:
             return None
-        ok, abs_ut1 = t.get_am_bcd(40, 41, 42, 43)
-        if not ok:
+        abs_ut1 = t._get_am_bcd(40, 41, 42, 43)
+        if abs_ut1 is None:
             return None
         abs_ut1 *= 100
         ut1_sign = t.am[38]
         ut1 = abs_ut1 if ut1_sign else -abs_ut1
-        ok, year = t.get_am_bcd(45, 46, 47, 48, 50, 51, 52, 53)
-        if not ok:
+        year = t._get_am_bcd(45, 46, 47, 48, 50, 51, 52, 53)
+        if year is None:
             return None
-        is_ly = t.am[55]
-        if days > 366 or (not is_ly and days > 365):
+        ly = bool(t.am[55])
+        if days > 366 or (not ly and days > 365):
             return None
-        ls = t.am[56]
-        _, dst = t.get_am_bcd(57, 58)
-
-        return cls(year, days, hour, minute, dst, ut1, ls)
+        ls = bool(t.am[56])
+        dst = require(t._get_am_bcd(57, 58))
+        return cls(year, days, hour, minute, dst, ut1, ls, ly)
 
 
 class WWVBMinuteIERS(WWVBMinute):
     """A WWVBMinute that uses a database of DUT1 information"""
 
     @classmethod
-    def get_dut1_info(cls, year, days, old_time=None):
+    def _get_dut1_info(
+        cls, year: int, days: int, old_time: Optional[WWVBMinute] = None
+    ) -> Tuple[int, bool]:
         d = datetime.datetime(year, 1, 1) + datetime.timedelta(days - 1)
         return int(round(get_dut1(d) * 10)) * 100, isls(d)
 
 
-def ilog10(v):
-    """Compute the integer log10 of a value"""
-    i = int(math.floor(math.log(v) / math.log(10)))
-    if 10 ** i > 10 * v:  # pragma no coverage
-        return i - 1
-    return i
-
-
-def bcd(n, d):
-    """Return the d'th bit of the BCD encoding of n"""
-    l = 10 ** ilog10(n)
-    n = (n // l) % 10
-    d = (d // l) % 10
-    return int(bool(n & d))
-
-
-bcd_weights = [1, 2, 4, 8, 10, 20, 40, 80, 100, 200, 400, 800]
+def bcd_bits(n: int) -> Generator[bool, None, None]:
+    """Return the bcd representation of n, starting with the least significant bit"""
+    while True:
+        d = n % 10
+        n = n // 10
+        for i in (1, 2, 4, 8):
+            yield bool(d & i)
 
 
 @enum.unique
@@ -683,62 +758,54 @@ class WWVBTimecode:
         self.am = [AmplitudeModulation.UNSET] * sz  # pylint: disable=invalid-name
         self.phase = [PhaseModulation.UNSET] * sz
 
-    @property
-    def data(self) -> list:
-        """An alias for `self.am`"""
-        return self.am
-
-    def get_am_bcd(self, *poslist):
+    def _get_am_bcd(self, *poslist: int) -> Optional[int]:
         """Convert the bits seq[positions[0]], ... seq[positions[len(positions-1)]] [in MSB order] from BCD to decimal"""
-        seq = self.am
-        pos = list(poslist)[::-1]
-        val = [int(seq[p]) for p in pos]
-        while len(val) % 4 != 0:
-            val.append(0)
+        pos = reversed(poslist)
+        val = [bool(self.am[p]) for p in pos]
         result = 0
         base = 1
         for i in range(0, len(val), 4):
             digit = 0
-            for j in range(4):
-                digit += 1 << j if val[i + j] else 0
+            for j, b in enumerate(val[i : i + 4]):
+                digit += b << j
             if digit > 9:
-                return False, None
+                return None
             result += digit * base
             base *= 10
-        return True, result
+        return result
 
-    def put_am_bcd(self, v: int, *poslist: Tuple[int]) -> None:
+    def _put_am_bcd(self, v: int, *poslist: int) -> None:
         """Treating 'poslist' as a sequence of indices, update the AM signal with the value as a BCD number"""
         pos = list(poslist)[::-1]
-        weights = bcd_weights[: len(pos)]
-        for p, w in zip(pos, weights):
-            b = bcd(w, v)
+        for p, b in zip(pos, bcd_bits(v)):
             if b:
                 self.am[p] = AmplitudeModulation.ONE
             else:
                 self.am[p] = AmplitudeModulation.ZERO
 
-    def put_pm_bit(self, i: int, v: bool) -> None:
+    def _put_pm_bit(self, i: int, v: Union[PhaseModulation, int, bool]) -> None:
         """Update a bit of the Phase Modulation signal"""
         self.phase[i] = PhaseModulation(v)
 
-    def put_pm_bin(self, st: int, n: int, v: bool) -> None:
+    def _put_pm_bin(self, st: int, n: int, v: int) -> None:
         """Update an n-digit binary number in the Phase Modulation signal"""
         for i in range(n):
-            self.put_pm_bit(st + i, extract_bit(v, (n - i - 1)))
+            self._put_pm_bit(st + i, extract_bit(v, (n - i - 1)))
 
     def __str__(self) -> str:
         """implement str()"""
-        undefined = [i for i in range(len(self.am)) if self.am[i] is None]
-        if undefined:  # pragma no coverage
-            print("Warning: Timecode%s is undefined" % undefined)
+        undefined = [
+            i for i in range(len(self.am)) if self.am[i] == AmplitudeModulation.UNSET
+        ]
+        if undefined:
+            warnings.warn(f"am{undefined} is unset")
 
-        def convert_one(am, phase):
+        def convert_one(am: AmplitudeModulation, phase: PhaseModulation) -> str:
             if phase is PhaseModulation.UNSET:
-                return ["0", "1", "2", "?"][am]
+                return ("0", "1", "2", "?")[am]
             if phase:
-                return ["⁰", "¹", "²", "?"][am]
-            return ["₀", "₁", "₂", "?"][am]
+                return ("⁰", "¹", "²", "¿")[am]
+            return ("₀", "₁", "₂", "⸮")[am]
 
         return "".join(convert_one(i, j) for i, j in zip(self.am, self.phase))
 
@@ -746,47 +813,92 @@ class WWVBTimecode:
         """implement repr()"""
         return "<WWVBTimecode " + str(self) + ">"
 
-    def to_am_string(self, charset: List[List[str]]) -> str:
+    def to_am_string(self, charset: List[str]) -> str:
         """Convert the amplitude signal to a string"""
         return "".join(charset[i] for i in self.am)
 
     to_string = to_am_string
 
-    def to_pm_string(self, charset: List[List[str]]) -> str:
+    def to_pm_string(self, charset: List[str]) -> str:
         """Convert the phase signal to a string"""
         return "".join(charset[i] for i in self.phase)
 
+    def to_both_string(self, charset: List[str]) -> str:
+        """Convert both channels to a string"""
+        return "".join(charset[i + j * 3] for i, j in zip(self.am, self.phase))
+
 
 styles = {
     "default": ["0", "1", "2"],
     "duration": ["2", "5", "8"],
     "cradek": ["0", "1", "-"],
     "bar": ["▟█", "▄█", "▄▟"],
+    "sextant": ["🬍🬎", "🬋🬎", "🬋🬍", "🬩🬹", "🬋🬹", "🬋🬩"],
 }
 
 
 # pylint: disable=too-many-arguments
-def print_timecodes(w, minutes, channel, style, file, *, all_timecodes=False):
+def print_timecodes(
+    w: WWVBMinute,
+    minutes: int,
+    channel: str,
+    style: str,
+    file: TextIO,
+    *,
+    all_timecodes: bool = False,
+) -> None:
     """Print a range of timecodes with a header.  This header is in a format understood by WWVBMinute.fromstring"""
-    channel_text = "" if channel == "amplitude" else " --channel=%s" % channel
-    style_text = "" if style == "default" else " --style=%s" % style
-    style = styles.get(style, "012")
+    channel_text = "" if channel == "amplitude" else f" --channel={channel}"
+    style_text = "" if style == "default" else f" --style={style}"
+    style_chars = styles.get(style, ["0", "1", "2"])
     first = True
     for _ in range(minutes):
         if first or all_timecodes:
             if not first:
                 print(file=file)
-            print(
-                "WWVB timecode: %s%s%s" % (str(w), channel_text, style_text), file=file
-            )
+            print(f"WWVB timecode: {str(w)}{channel_text}{style_text}", file=file)
         first = False
-        pfx = "%04d-%03d %02d:%02d " % (w.year, w.days, w.hour, w.min)
+        pfx = f"{w.year:04d}-{w.days:03d} {w.hour:02d}:{w.min:02d} "
+        tc = w.as_timecode()
+        if len(style_chars) == 6:
+            print(f"{pfx} {tc.to_both_string(style_chars)}", file=file)
+            print(file=file)
+            pfx = " " * len(pfx)
+        else:
+            if channel in ("amplitude", "both"):
+                print(f"{pfx} {tc.to_am_string(style_chars)}", file=file)
+                pfx = " " * len(pfx)
+            if channel in ("phase", "both"):
+                print(f"{pfx} {tc.to_pm_string(style_chars)}", file=file)
+            if channel == "both":
+                print(file=file)
+        w = w.next_minute()
+
+
+# pylint: disable=too-many-arguments
+def print_timecodes_json(
+    w: WWVBMinute,
+    minutes: int,
+    channel: str,
+    file: TextIO,
+) -> None:
+    """Print a range of timecodes with a header.  This header is in a format understood by WWVBMinute.fromstring"""
+    result = []
+    for _ in range(minutes):
+        data = {
+            "year": w.year,
+            "days": w.days,
+            "hour": w.hour,
+            "minute": w.min,
+        }
+
         tc = w.as_timecode()
         if channel in ("amplitude", "both"):
-            print("%s %s" % (pfx, tc.to_am_string(style)), file=file)
-            pfx = " " * len(pfx)
+            data["amplitude"] = tc.to_am_string(["0", "1", "2"])
         if channel in ("phase", "both"):
-            print("%s %s" % (pfx, tc.to_pm_string(style)), file=file)
-        if channel == "both":
-            print(file=file)
+            data["phase"] = tc.to_pm_string(["0", "1"])
+
+        result.append(data)
         w = w.next_minute()
+    json.dump(result, file)
+    print(file=file)
