@@ -12,13 +12,17 @@ import io
 import itertools
 import os
 import pathlib
-import bs4
+from typing import Callable, List, Optional
+import bs4  # type: ignore
 import click
 import platformdirs
 import requests
 
-DIST_PATH = str(pathlib.Path(__file__) / "iersdata_dist.py")
 
+DIST_PATH = str(pathlib.Path(__file__).parent / "iersdata_dist.py")
+
+OLD_TABLE_START: Optional[datetime.date] = None
+OLD_TABLE_END: Optional[datetime.date] = None
 try:
     import wwvb.iersdata_dist
 
@@ -27,18 +31,17 @@ try:
         days=len(wwvb.iersdata_dist.DUT1_OFFSETS) - 1
     )
 except (ImportError, NameError) as e:
-    OLD_TABLE_START = OLD_TABLE_END = None
-
+    pass
 IERS_URL = "https://datacenter.iers.org/data/csv/finals2000A.all.csv"
 NIST_URL = "https://www.nist.gov/pml/time-and-frequency-division/atomic-standards/leap-second-and-ut1-utc-information"
 
 
-def update_iersdata(
-    target_file,
-):  # pylint: disable=too-many-locals, too-many-branches, too-many-statements
+def update_iersdata(  # pylint: disable=too-many-locals, too-many-branches, too-many-statements
+    target_file: str,
+) -> None:
     """Update iersdata.py"""
 
-    offsets = []
+    offsets: List[int] = []
     with requests.get(IERS_URL) as iers_data:
         for r in csv.DictReader(io.StringIO(iers_data.text), delimiter=";"):
             jd = float(r["MJD"])
@@ -71,18 +74,20 @@ def update_iersdata(
         .date()
     )
 
-    def patch(patch_start, patch_end, val):
+    def patch(patch_start: datetime.date, patch_end: datetime.date, val: int) -> None:
         off_start = (patch_start - table_start).days
         off_end = (patch_end - table_start).days
         offsets[off_start:off_end] = [val] * (off_end - off_start)
 
-    wwvb_dut1 = None
+    wwvb_dut1: Optional[int] = None
+    wwvb_start: Optional[datetime.date] = None
     for row in wwvb_dut1_table.findAll("tr")[1:][::-1]:
         cells = row.findAll("td")
         when = datetime.datetime.strptime(cells[0].text, "%Y-%m-%d").date()
         dut1 = cells[2].text.replace("s", "").replace(" ", "")
         dut1 = int(round(float(dut1) * 10))
         if wwvb_dut1 is not None:
+            assert wwvb_start is not None
             patch(wwvb_start, when, wwvb_dut1)
         wwvb_dut1 = dut1
         wwvb_start = when
@@ -96,11 +101,25 @@ def update_iersdata(
     # this is the final (most recent) wwvb DUT1 value broadcast.  We want to
     # extend it some distance into the future, but how far?  We will use the
     # modified timestamp of the NIST data.
+    assert wwvb_dut1 is not None
+    assert wwvb_start is not None
+
+    # As of 2021-12-17, NIST website incorrectly indicates the offset of -200ms
+    # persisted through present.  However, based on private correspondance and
+    # IERS Bulletin D 141 <https://datacenter.iers.org/data/html/bulletind-141.html>
+    # it appears that -100ms should have been broadcast starting on 2021-07-17.
+    gap_start = datetime.date(2021, 7, 17)
+    if wwvb_start < gap_start:
+        patch(wwvb_start, gap_start, wwvb_dut1)
+        wwvb_start = gap_start
+        wwvb_data_stamp = datetime.date(2021, 12, 17)
+        wwvb_dut1 = -1
+
     patch(wwvb_start, wwvb_data_stamp + datetime.timedelta(days=1), wwvb_dut1)
 
     with open(target_file, "wt", encoding="utf-8") as output:
 
-        def code(*args):
+        def code(*args: str) -> None:
             """Print to the output file"""
             print(*args, file=output)
 
@@ -113,35 +132,34 @@ def update_iersdata(
         code("import datetime")
 
         code("__all__ = ['DUT1_DATA_START', 'DUT1_OFFSETS']")
-        code("DUT1_DATA_START = %r" % table_start)
+        code(f"DUT1_DATA_START = {repr(table_start)}")
         c = sorted(chr(ord("a") + ch + 10) for ch in set(offsets))
-        code("%s = '%s'" % (",".join(c), "".join(c)))
+        code(f"{','.join(c)} = tuple({repr(''.join(c))})")
         code(
-            "DUT1_OFFSETS = str( # %04d%02d%02d"
-            % (table_start.year, table_start.month, table_start.day)
+            f"DUT1_OFFSETS = str( # {table_start.year:04d}{table_start.month:02d}{table_start.day:02d}"
         )
         line = ""
         j = 0
 
-        for ch, it in itertools.groupby(offsets):
+        for val, it in itertools.groupby(offsets):
             part = ""
-            ch = chr(ord("a") + ch + 10)
+            ch = chr(ord("a") + val + 10)
             sz = len(list(it))
             if j:
                 part = part + "+"
             if sz < 2:
-                part = part + "%s" % ch
+                part = part + ch
             else:
-                part = part + "%s*%d" % (ch, sz)
+                part = part + f"{ch}*{sz}"
             j += sz
             if len(line + part) > 60:
                 d = table_start + datetime.timedelta(j - 1)
-                code("    %-60s # %04d%02d%02d" % (line, d.year, d.month, d.day))
+                code(f"    {line:<60s} # {d.year:04d}{d.month:02d}{d.day:02d}")
                 line = part
             else:
                 line = line + part
         d = table_start + datetime.timedelta(j - 1)
-        code("    %-60s # %04d%02d%02d" % (line, d.year, d.month, d.day))
+        code(f"    {line:<60s} # {d.year:04d}{d.month:02d}{d.day:02d}")
         code(")")
     table_end = table_start + datetime.timedelta(len(offsets) - 1)
     if OLD_TABLE_START:
@@ -149,9 +167,9 @@ def update_iersdata(
     print(f"iersdata covers {table_start} .. {table_end}")
 
 
-def iersdata_path(callback):
+def iersdata_path(callback: Callable[[str, str], str]) -> str:
     """Find out the path for this directory"""
-    return callback("wwvbpy", "unpythonic.net")
+    return os.path.join(callback("wwvbpy", "unpythonic.net"), "wwvb_iersdata.py")
 
 
 @click.command()
@@ -165,11 +183,11 @@ def iersdata_path(callback):
 @click.option(
     "--site", "location", flag_value=iersdata_path(platformdirs.site_data_dir)
 )
-def main(location):
+def main(location: str) -> None:
     """Update DUT1 data"""
     print("will write to", location)
-    os.makedirs(location, exist_ok=True)
-    update_iersdata(os.path.join(location, "wwvbpy_iersdata.py"))
+    os.makedirs(os.path.dirname(location), exist_ok=True)
+    update_iersdata(location)
 
 
 if __name__ == "__main__":
